from .api.instagram_private_api.errors import (
    ClientError, ClientLoginError, ClientLoginRequiredError,
    ClientCookieExpiredError, ClientThrottledError, ClientConnectionError,
    ClientCheckpointRequiredError, ClientChallengeRequiredError,
    ClientSentryBlockError, ClientReqHeadersTooLargeError,
)


class Stop(Exception):
    pass